﻿// 
//       Copyright (C) DataStax Inc.
// 
//    Licensed under the Apache License, Version 2.0 (the "License");
//    you may not use this file except in compliance with the License.
//    You may obtain a copy of the License at
// 
//       http://www.apache.org/licenses/LICENSE-2.0
// 
//    Unless required by applicable law or agreed to in writing, software
//    distributed under the License is distributed on an "AS IS" BASIS,
//    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//    See the License for the specific language governing permissions and
//    limitations under the License.

using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;

namespace Cassandra.Tests.TestHelpers
{
    public static class TestHostFactory
    {
        public static Host Create(Guid? hostId = null, string ipAddress = "127.0.0.1", string dc = "dc1", string rack = "rack1", IEnumerable<string> tokens = null)
        {
            hostId = hostId ?? Guid.NewGuid();
            var row = new TestHelper.DictionaryBasedRow(new Dictionary<string, object>
                {
                    { "host_id", hostId },
                    { "data_center", dc},
                    { "rack", rack },
                    { "release_version", "3.11.1" },
                    { "tokens", tokens?.ToList() ?? new List<string> { "1" }}
                });
            var host = new Host(new IPEndPoint(IPAddress.Parse(ipAddress), 9042), contactPoint: null);
            host.SetInfo(row);
            return host;
        }
    }
}