﻿//
//      Copyright (C) DataStax Inc.
//
//   Licensed under the Apache License, Version 2.0 (the "License");
//   you may not use this file except in compliance with the License.
//   You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
//   Unless required by applicable law or agreed to in writing, software
//   distributed under the License is distributed on an "AS IS" BASIS,
//   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//   See the License for the specific language governing permissions and
//   limitations under the License.
//

using System;

namespace Cassandra
{
    [Flags]
    internal enum HeaderFlags : byte
    {
        None = 0,

        /// <summary>
        /// If set, the frame body is compressed.
        /// </summary>
        Compression = 0x01,

        /// <summary>
        /// For a request frame, this indicate the client requires tracing of the request.
        /// If a response frame has the tracing flag set, its body contains a tracing ID.
        /// </summary>
        Tracing = 0x02,

        /// <summary>
        /// For a request or response frame, this indicates that generic key-value 
        /// custom payload for a custom QueryHandler implementation is present in the frame.
        /// </summary>
        CustomPayload = 0x04,

        /// <summary>
        /// The response contains warnings from the server which 
        /// were generated by the server to go along with this response.
        /// </summary>
        Warning = 0x08,
        
        /// <summary>
        /// Indicates that the client opts in to use protocol version
        /// that is currently in beta. Server will respond with ERROR if protocol
        /// version is marked as beta on server and client does not provide this flag.
        /// </summary>
        UseBeta = 0x10
    }
}
